from typing import TypedDict, Any
import json
import requests
import logging
from .base_api_wrapper import RestAPIWrapper
from ..models_configurations.dalle_config import DALLEConfiguration
from ..wrapper_interfaces.file_generating_model_wrapper import FileGenModelWrapper

IMAGE_GENARATION_API_URL = "https://api.openai.com/v1/images/generations"


class ImageGenerationRequestBody(TypedDict):
    prompt: str
    n: int
    size: str


class DALLEWrapper(RestAPIWrapper, FileGenModelWrapper):
    config: DALLEConfiguration
    
    def __init__(self, configuration: DALLEConfiguration, logger: logging.Logger=logging.getLogger()) -> None:
        super().__init__(configuration, logger)
        self.api_url = IMAGE_GENARATION_API_URL
    
    def _get_body(self, prompt_message: str) -> Any:
        return json.dumps({
            "prompt": prompt_message,
            "n": self.config.num_of_images,
            "size": self.config.resolution
        })

    def _get_api_url(self) -> str:
        return self.api_url

    def prompt_and_generate_files(self, prompt: str) -> Any:
        response = self.prompt(prompt)

        images = []
        images_path = []
        data = response['data']

        for idx, item in enumerate(data):
            response = requests.get(item['url'])
            images.append(response.content)

            with open(f"image_{idx}.jpg", "wb") as f:
                f.write(response.content)
            images_path.append(f"image_{idx}.jpg")

        return {'images': images,
                'images_path': images_path}



from typing import Any
import json
import logging
from .base_api_wrapper import RestAPIWrapper
from ..models_configurations.hugging_face_config import HuggingFaceConfiguration

HUGGING_FACE_BASE_URL = "https://api-inference.huggingface.co/models"


class HuggingFaceWrapper(RestAPIWrapper):
    config: HuggingFaceConfiguration

    def __init__(self, configuration: HuggingFaceConfiguration, logger: logging.Logger=logging.getLogger()) -> None:
        super().__init__(configuration, logger)
        self.api_url = HUGGING_FACE_BASE_URL

    def _get_body(self, prompt_message: str) -> Any:
        return json.dumps({
            "inputs": prompt_message,
        })

    def _get_api_url(self) -> str:
        return f"{self.api_url}/{self.config.model_id}"



from .base_config import BaseConfiguration
from .config_types import ConfigTypes


class DALLEConfiguration(BaseConfiguration):
    def __init__(self, api_key: str, num_of_images: int, resolution: str) -> None:
        super().__init__(api_key)
        self.num_of_images = num_of_images
        self.resolution = resolution

    def _get_config_type(self) -> ConfigTypes:
        return ConfigTypes.DALLE



from omnibridge.wrappers.api_based_wrappers.gpt_wrapper import GPTWrapper
from omnibridge.wrappers.api_based_wrappers.dalle_wrapper import DALLEWrapper
from omnibridge.wrappers.api_based_wrappers.hugging_face_wrapper import HuggingFaceWrapper
from omnibridge.wrappers.models_configurations.base_config import BaseConfiguration
from omnibridge.wrappers.models_configurations.chatgpt_config import GPTConfiguration
from omnibridge.wrappers.models_configurations.dalle_config import DALLEConfiguration
from omnibridge.wrappers.models_configurations.hugging_face_config import HuggingFaceConfiguration
from omnibridge.common.logging.log_manager import LogManager
import os
from typing import Any, cast

logger = LogManager().logger

def run_prompt_in_chatgpt_wrapper(prompt: str, config: BaseConfiguration | None = None) -> Any:
    if config:
        config = cast(GPTConfiguration, config)
    else:
        config = GPTConfiguration(model='gpt-3.5-turbo', 
                                  api_key=os.getenv("OPENAI_API_KEY", ""))
        
    wrapper = GPTWrapper(configuration=config)

    try:
        response = wrapper.prompt_and_get_response(prompt)
        print(response)
        return response
    except Exception as e:
        print(e)

    return None


def run_prompt_in_dalle_wrapper(prompt: str, config: BaseConfiguration | None = None) -> Any:
    if config:
        config = cast(DALLEConfiguration, config)
    else:
        config = DALLEConfiguration(api_key=os.getenv("OPENAI_API_KEY", ""),
                                                       resolution='256x256',
                                                       num_of_images=4)
        
    wrapper = DALLEWrapper(configuration=config, logger=logger)

    try:
        ret = wrapper.prompt_and_generate_files(prompt)
        return ret
    except Exception as e:
        print(e)

    return None


def run_prompt_in_hugging_face_wrapper(prompt: str, config: BaseConfiguration | None = None) -> Any:
    if config:
        config = cast(HuggingFaceConfiguration, config)
    else:
        config = HuggingFaceConfiguration(api_key=os.getenv("HUGGING_FACE_API_KEY", ""), 
                                          model_id="distilbert-base-uncased")

    wrapper = HuggingFaceWrapper(configuration=config, logger=logger)

    try:
        ret = wrapper.prompt(prompt)
        return ret
    except Exception as e:
        print(e)

    return None



import logging
from datetime import datetime
from typing import Any

class LogManager:
    """
    LogManager is a centralized logging class that configures and manages logging for an application.
    It sets up a logger with both stream and file handlers, allowing log messages to be output to the console
    and a file simultaneously. LogManager uses a singleton pattern to ensure only one instance is created
    throughout the application.

    Attributes:
        FORMATTER (logging.Formatter): Formatter for the log messages.
        STREAM_LOG_LEVEL (int): The log level for the stream handler (default: logging.ERROR).
        FILE_LOG_LEVEL (int): The log level for the file handler (default: logging.DEBUG).
        LOG_FILE_NAME (str): The name of the log file with the current date.
        LOGGER_NAME (str): The name of the logger.

    Example:
        log_manager = LogManager()
        logger = log_manager.logger

        logger.debug("This is a debug message.")
        logger.info("This is an info message.")
        logger.warning("This is a warning message.")
        logger.error("This is an error message.")
        logger.critical("This is a critical message.")
    """
    _instance: Any = None
    _initialized: bool = False

    FORMATTER = logging.Formatter('%(asctime)s [%(levelname)s]: %(message)s')
    STREAM_LOG_LEVEL = logging.ERROR
    FILE_LOG_LEVEL = logging.DEBUG
    LOG_FILE_NAME = ".log-" + datetime.today().strftime('%d-%m-%Y')
    LOGGER_NAME = "OmniSpectiveLogger"

    def __new__(cls: Any) -> Any:
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._instance._initialized = False
        return cls._instance

    def __init__(self, stream_log_level: int = STREAM_LOG_LEVEL) -> None:
        if self._initialized:
            return
        
        logger = logging.getLogger(LogManager.LOGGER_NAME)
        logger.setLevel(logging.DEBUG)

        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(stream_log_level)
        stream_handler.setFormatter(LogManager.FORMATTER)

        file_handler = logging.FileHandler(LogManager.LOG_FILE_NAME)
        file_handler.setLevel(LogManager.FILE_LOG_LEVEL)
        file_handler.setFormatter(LogManager.FORMATTER)

        logger.addHandler(stream_handler)
        logger.addHandler(file_handler)

        self._logger = logger

        self._initialized = True

    @property
    def logger(self) -> logging.Logger:
        return self._logger



from typing import Any
from abc import ABC, abstractmethod
from .config_types import ConfigTypes


class BaseConfiguration(ABC):
    def __init__(self, api_key: str) -> None:
        self.api_key = api_key

    def to_dict(self) -> dict[str, Any]:
        return self.__dict__

    @abstractmethod
    def _get_config_type(self) -> ConfigTypes:
        pass



from typing import Any
import requests
from abc import ABC, abstractmethod
from ..models_configurations.base_config import BaseConfiguration
from ..wrapper_interfaces.textual_model_wrapper import TextualModelWrapper

import logging


class WrapperException(Exception):
    pass


class RestAPIWrapper(ABC):
    def __init__(self, configuration: BaseConfiguration, logger: logging.Logger = logging.getLogger()) -> None:
        self.config = configuration
        self.logger = logger

    def _get_headers(self) -> dict[str, str]:
        return {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.config.api_key}"
        }

    @abstractmethod
    def _get_body(self, prompt_message: str) -> Any:
        pass

    @abstractmethod
    def _get_api_url(self) -> str:
        pass

    def prompt(self, prompt_message: str) -> Any:
        """
        raises GPTWrapperException if request failed
        returns string response from chatgpt completions api
        """
        self.logger.debug(f'Sending prompt to API: {prompt_message}')
        response = requests.post(
            self._get_api_url(),
            headers=self._get_headers(),
            data=self._get_body(prompt_message)
        )

        try:
            response.raise_for_status()
        except Exception as e:
            error_message = f"Request to api endpoint: {self._get_api_url()} failed.\n" \
                            f"Response message: {response.text}.\n" \
                            f"Exception caught: {e}"
            self.logger.error(error_message)
            raise WrapperException(error_message)

        return response.json()


class TextualRestAPIWrapper(RestAPIWrapper, TextualModelWrapper):
    @abstractmethod
    def _parse_response(self, response_json: Any) -> str:
        pass

    def prompt_and_get_response(self, prompt: str) -> str:
        return self._parse_response(self.prompt(prompt))



from .base_config import BaseConfiguration
from .config_types import ConfigTypes
from typing import TypeVar, Literal

chatGptModel = TypeVar('chatGptModel', bound=Literal["gpt-4", "gpt-4-0314",
                                                      "gpt-4-32k", "gpt-4-32k-0314",
                                                        "gpt-3.5-turbo", "gpt-3.5-turbo-0301"])

class GPTConfiguration(BaseConfiguration):
    def __init__(self, api_key: str, model: chatGptModel) -> None:
        super().__init__(api_key)
        self.model = model

    def _get_config_type(self) -> ConfigTypes:
        return ConfigTypes.CHATGPT